package LCFG::Config;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

use LCFG::Metadata;
use LCFG::Resource;
use LCFG::Property;
use LCFG::StructResources;
use LCFG::Path;

#require Exporter;
#require AutoLoader;

#@ISA = qw(Exporter AutoLoader);
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
#@EXPORT = qw();

$VERSION = '1.0.10';


# Preloaded methods go here.

sub new {
  my $proto = shift;
  my $class = ref($proto) || $proto;
  my $self = {};
  bless($self,$class);

  if ($self->_initialize(@_)) {
    return $self;
  } else {
    undef $self;
    return undef;
  }
}

sub _initialize {
  my ($self,$res)=@_;
  
  my $structres=LCFG::StructResources->new($res);
  unless (defined $structres) {
    warn "cannot instantiate StructResources";
    return undef;
  }
  $self->{'S_RES'}=$structres;
  return 1;
}



sub release {
  my $self = shift;
  warn "method not implemented";
  return undef;
}

sub getConfigId {
  my $self = shift;
  warn "method not implemented";
  return undef;
}

sub getMetadata {
  my ($self,$path)=@_;

  unless (ref $path) { # is it a string with the path or a path object?
    $path=LCFG::Path->new($path);
  }

  my $element = $self->{'S_RES'}->getpbyname($path->toString());
  return undef unless (defined $element);
  return LCFG::Metadata->new($element);
}

sub getElement {
  my ($self,$path) = @_;

  unless (ref $path) { # is it a string with the path or a path object?
    $path=LCFG::Path->new($path);
  }

  my $element = $self->{'S_RES'}->getpbyname($path->toString());
  return undef unless (defined $element);
  my $metadata = LCFG::Metadata->new($element);
  if ($metadata->getType() =~ /^resource$/i) {
    return LCFG::Resource->new($element,$metadata);
  } else {
    return LCFG::Property->new($element,$metadata);
  }
}


sub getValue {
  my ($self,$path) =@_;
  
  my $element=$self->getElement($path);
  
  return $element->getValue();
}
  


sub lock {
  my $self = shift;
  warn "method not implemented";
  return undef;
}

sub unLock {
  my $self = shift;
  warn "method not implemented";
  return undef;
}

sub isLocked {
  my $self = shift;
  warn "method not implemented";
  return undef;
}



1;
__END__

=head1 NAME

LCFG::Config - An LCFG configuration object

=head1 SYNOPSIS

 $config->release();
 $configid = $config->getConfigId();
 $metadata = $config->getMetadata($path);
 $element = $config->getElement($path);
 $config->lock();
 $config->unLock();
 $boolean = $config->isLocked();

 Core API extensions:
 $metadata = $config->getMetadata($path_string);
 $element = $config->getElement($path_string);

=head1 DESCRIPTION

Provides an API to the configuration object. This object is used to
retrieve elements from the node profile for a given path. It can also
be used to lock/unlock the configuration.

=head2 METHODS

=over 4

=item release()

Release the configuration. The cache manager (see L<LCFG::CacheManager>) is
notified that this particular configuration is no longer in use. A
configuration object should not be used again after it has been released.

(Not implemented in this version)

=item getConfigId()

Returns the numeric configuration identifier. This can then be used by child
processes to open the same configuration as their parent.

(Not implemented in this version)

=item getMetadata($path)

Returns the metadata for the element corresponding to the given path (a path
object, see L<LCFG::Path>) in the node profile. The Metadata interface can
be used to access individual attributes, see L<LCFG::Metadata>.

=item getElement($path)

Returns a resource (see L<LCFG::Resource>) or property (see L<LCFG::Property>)
element object corresponding to the given path (a path object,
see L<LCFG::Path>) in the node profile.

=item lock()

Lock the configuration. If the configuration is locked any updates to the
configuration will not be visible.

(Not implemented in this version)

=item unLock()

Unlock the configuration. If the configuration is unlocked any updates to
the configuration may become immediately visible.

(Not implemented in this version)

=item isLocked()

Returns true if the configuration is locked and false otherwise.

(Not implemented in this version)

=item Core API extensions:

=item getMetadata($path_string)

Returns the corresponding LCFG::Metadata instance.

=item getElement($path_string)

Returns the corresponding LCFG::{Resource or Property} instance.

=item getValue($path) [non official API extension]

Returns the value of the corresponding element


=head1 EXAMPLES

 use LCFG::CacheManager;
 use LCFG::Path;

 # create a connection to a cache and get a configuration 
 $cache = LCFG::CacheManager->new();
 $config = $cache->getUnLockedConfig();

 # lock the configuration

 $config->lock();

 # create a path object to an element
 $path = LCFG::Path->new('/test');

 # get the element from the configuration corresponding to the path
 $element = $config->getElement($path);

 # unlock and release the configuration
 $config->unlock();
 $config->release();

=head1 AUTHOR

Original author: Tim Colles, timc@dai.ed.ac.uk

Adapted to LCFG::StructResources, core API extensions: 
German Cancio <German.Cancio@cern.ch>


=head1 SEE ALSO

LCFG::StructResources

=cut
