package LCFG::Path;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

#require Exporter;
#require AutoLoader;

#@ISA = qw(Exporter AutoLoader);
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
#@EXPORT = qw();
$VERSION = '1.0.10';


# Preloaded methods go here.

my $pathSep = '/';

sub new {
  my $proto = shift;
  my $class = ref($proto) || $proto;
  my $self = {};
  $self->{PATH} = [];
  if (@_) {
    my $tmp = shift;
    $tmp =~ s/^$pathSep//;
    down($self,$tmp);
  }
  bless($self,$class);
  return $self;
}

sub toString {
  my $self = shift;
  return $pathSep.join($pathSep,@{$self->{PATH}});
}

sub down {
  my $self = shift;
  if (@_) {
    my @items = split($pathSep,shift);
    foreach my $item ( @items ) {
      if ($item !~ /^(\w|-)+$/) {
        die "Wrong Path Exception: $item";
      }
      else {
        push @{$self->{PATH}},$item;
      }
    }
  }
}

sub up {
  my $self = shift;
  pop @{$self->{PATH}}
}

sub initPath {
  my $path = shift;
  return LCFG::Path->new($path);
}

# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__
# Below is the stub of documentation for your module. You better edit it!

=head1 NAME

LCFG::Path - An LCFG node profile path object

=head1 SYNOPSIS

 use LCFG::Path;

 $path = LCFG::Path->new($stringpath);
 $path = initPath($stringpath);

 $path->down($stringpath);
 $path->up();
 $stringvalue = $path->toString();

=head1 DESCRIPTION

Provides an object representing a path in a node profile and provides an
API to create and manipulate that path. The path object can then be used
with the configuration to get the actual resource or property element
corresponding to the path. This API only constructs a path, it does not
check it for validity.

=head2 CONSTRUCTOR

=over 4

=item LCFG::Path->new($stringpath), initPath($stringpath)

Create a new path object using stringpath as the string representation of the
path. The path can be given as absolute or relative, in the latter case
it is assumed to be relative to root on construction. The standard path
separator is '/'. Relative path notation '.' and '..' not supported. If no
argument is given a path object to root is created.

=head2 METHODS

=over 4

=item down($stringpath)

Modifies the path object by concatenation of the current path and stringpath
(which follows the same path syntax as in construction) which must be a
relative path. No change is made to the path object if no argument is given.

=item up()

Modifies the path object by removing the last element from the path. Can be
repeatedly applied (when the path is reduced to root it no longer has any
affect).

= item toString()

Returns the path object as a string path using the standard separator.

=head1 EXCEPTIONS

=item * WRONG PATH

Raised if a stringpath argument during construction or on down() has an
invalid syntax. A stringpath must consist of elements separated by the
'/' character. Multiple '/' characters without a separating element are
illegal as are trailing '/' characters. Each element can contain alphanumeric
characters, the dash character and the underscore character but must start
with an alphabetic character. A '.' or '..' sequence is considered to be an
element and is hence illegal as well.

=head1 EXAMPLES

 use LCFG::Path;

 # create a path object
 $path = LCFG::Path->new('/test/sub');

 # go down another couple of elements
 $path->down('sub2/sub3');

 # go back up one element
 $path->up();

 # print the current path as a string
 print $path->toString()."\n";

=head1 AUTHOR

Tim Colles, timc@dai.ed.ac.uk

=head1 SEE ALSO

This API is part of the LCFG API, see L<LCFG::CacheManager> for
further information.

=cut
