package LCFG::Property;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

use LCFG::Element;

#require Exporter;
#require AutoLoader;

#@ISA = qw(Exporter AutoLoader LCFG::Element);
@ISA = qw(LCFG::Element);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
#@EXPORT = qw();
$VERSION = '1.0.10';


# Preloaded methods go here.

sub new {
  my $proto = shift;
  my $class = ref($proto) || $proto;
  my $self = {};
  bless($self,$class);

  if ($self->_initialize(@_)) {
    return $self;
  } else {
    undef $self;
    return undef;
  }
}


sub _initialize {
  my $self=shift;

  unless ($self->SUPER::_initialize(@_)) {
    return undef;
  }
  # take the value...
  $self->{'VALUE'}=$self->{'ELEMENT'}->{'VALUE'};

  return 1;
}




sub getStringValue {
  my $self = shift;
  if ($self->{METADATA}->getType() !~ /^string$/i) {
    warn "Wrong Mapping Exception";
    return undef;
  } else {
    return $self->{VALUE};
  }
}

sub getBooleanValue {
  my $self = shift;
  if ($self->{METADATA}->getType() !~ /^boolean$/i) {
    warn "Wrong Mapping Exception";
    return undef;
  } else {
    if ($self->{VALUE} =~ /^(1|yes|true|on)$/) {
      return 1;
    } elsif ($self->{VALUE} =~ /^(0|no|false|off)$/) {
      return 0;
    } else {
# Profile validation should ensure we never get here
      warn "Unrecognised boolean value: $self->{VALUE}";
      return undef;
    }
  }
}

sub getIntValue {
  my $self = shift;
  if ($self->{METADATA}->getType() !~ /^int$/i) {
    warn "Wrong Mapping Exception";
    return undef;
  } else {
    return $self->{VALUE};
  }
}

sub getLongValue {
  my $self = shift;
  if ($self->{METADATA}->getType() !~ /^long$/i) {
    warn "Wrong Mapping Exception";
    return undef;
  } else {
    return $self->{VALUE};
  }
}

sub getFloatValue {
  my $self = shift;
  if ($self->{METADATA}->getType() !~ /^float$/i) {
    warn "Wrong Mapping Exception";
    return undef;
  } else {
    return $self->{VALUE};
  }
}

sub getDoubleValue {
  my $self = shift;
  if ($self->{METADATA}->getType() !~ /^double$/i) {
    warn "Wrong Mapping Exception";
    return undef;
  } else {
    return $self->{VALUE};
  }
}

sub getBLOB {
  my $self = shift;
  if ($self->{METADATA}->getType() !~ /^blob$/i) {
    warn "Wrong Mapping Exception";
    return undef;
  } else {
    return $self->{VALUE};
  }
}

sub getStream {
  my $self = shift;
  if ($self->{METADATA}->getType() !~ /^stream$/i) {
    warn "Wrong Mapping Exception";
    return undef;
  } else {
    warn "method not implemented";
    return undef;
  }
}


sub getValue {
  my $self = shift;
  return $self->{VALUE};
}


# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__

=head1 NAME

LCFG::Property - An LCFG node profile property element

=head1 SYNOPSIS

 $stringvalue = $property->getStringValue();
 $numbervalue = $property->getBooleanValue();
 $numbervalue = $property->getIntValue();
 $numbervalue = $property->getLongValue();
 $numbervalue = $property->getFloatValue();
 $stringvalue = $property->getBLOB();
 $stream = $property->getStream(); # not implemented in this version

=head1 DESCRIPTION

LCFG::Property extends L<LCFG::Element>.

Provides an API to retrieve the value of a node profile property
element. The property object must already have been retrieved via the
configuration API.

=head2 METHODS

=over 4

=item getStringValue()

Returns the value of the property element as a string.

=item getBooleanValue()

Returns the value of the property element as a number which will have
the value one or zero where one corresponds to an actual property
value of 1, true, yes or on and zero corresponds to an actual property
value of 0, false, no or off.

=item getIntValue(), getLongValue(), getFloatValue(), getDoubleValue()

All return the value of the property element as a number.

=item getBLOB()

Returns the value of the property element as a string.

=item getStream()

The property value is streamed through the cache manager and the API. So
this returns a stream object which can be used to handle the streamed
data. See L<LCFG::Stream> for details on this.

=head1 Core API extensions

=item getValue() [non official API extension]

Returns the value of the element.


=head1 EXCEPTIONS

=item * WRONG MAPPING

Raised if the type value of the property being retrieved is not the same
as the type value of the calling method. For example, using getStringValue()
for a property whose type is not a string.

=head1 EXAMPLES

 use LCFG::CacheManager;
 use LCFG::Path;

 # create a connection to a cache and a configuration 
 $cache = LCFG::CacheManager->new();
 $config = $cache->getUnLockedConfig();

 # create a path object to a property element
 $path = LCFG::Path->new('/test/fubar');

 # get the property element corresponding to the path
 $property = $config->getElement($path);

 # get and display the value of the property
 print $property->getStringValue()."\n";

=head1 AUTHOR

Original author: Tim Colles, timc@dai.ed.ac.uk

Adapted to rdxprof and LCFG::StructResources: German Cancio <German.Cancio@cern.ch>


=head1 SEE ALSO

LCFG::StructResources, LCFG::Element

=cut
