#
# LCFG::StructResources class
#
# Copyright (c) 2001 EU DataGrid.
# For license conditions see http://www.eu-datagrid.org/license.html
#


package LCFG::StructResources;

use strict;
use LCFG::Resources;


=pod

=head1 NAME

LCFG::StructResources - provides an 'unflattened' resource structure

=head1 SYNOPSIS

  use LCFG::StructResources;

  my $res=LCFG::Resources(...);

  my $structres=LCFG::StructResources->new($res);

  $structres->...();


=head1 INHERITANCE

  None. Uses LCFG::Resources.

=head1 DESCRIPTION


 Note: only a subset of resources is accepted:
  1 the '_' character is reserved (cannot be used inside resource names)
  2 the resource placeholders (_$) have to go at the end of the resource name
  3 all resource hierarchies have to be contained in a sub-resource
  4 LCFGng default resources 'ng_*' and 'om_*' are dropped due to (1).

=cut



=pod

=back

=head2 Private methods

=over 4

=item _initialize(): called by new(), initializes the object internals.

=cut



sub _initialize {
  my ($self,$flatres)=@_;

  # internal (instance) variables

  $self->{'ELEMENT_REGISTRY'}={};
  
  my %root_element;
  $root_element{'NAME'}='/'; # this is the root element
  $root_element{'TYPE'}='resource'; #type: resource (interior element)
  $self->{'STRUCTRES'}=\%root_element;


  unless ($self->_convert2struct($flatres)) {
    return undef;
  }

  return 1; #success
}


=pod

=item _convert2struct($res)

_convert2struct creates the unflattened resource structure used by
all the other object methods.

=cut



sub _convert2struct {
  my ($self,$comp)=@_;



  my $comp_p = $self->{'STRUCTRES'};

  my %element_registry;

  my $comp_name;

  my $ok=1;

  my $flatres_name;
  my $flatres;
  
  my $changes;

  my %resources_to_process;

  my $nr_resources_process=0;

    #
    # find out which resources are 'structure' resources
    #
  foreach (keys(%{$comp})) {
    next if ($_ =~ m%^(ng|om)_%); # ignore LCFG internal ng_ and om_ resources
    my $type=$comp->{$_}->{'TYPE'};
    if (defined $type && $type =~ m%^list\:%) {
      $resources_to_process{$_}=1;
      $nr_resources_process++;
    }
  }
  
  
  #
  # First: process 'structure' resources
  #
  
  do {
    
    $changes=0;
    
    #print "structure resources to go: $nr_resources_process\n";
    
    unless ($nr_resources_process==0) {
      
      foreach $flatres_name (keys(%{$comp})) {
	# loop over the resources of the component
	
	next unless ($resources_to_process{$flatres_name}==1);
	# care only about the (structure) ones to process
	
	my $flatres = $comp->{$flatres_name};
	my $resource_name=$flatres_name;
	$resource_name =~ s%([^_]+)(\_.+)%$1%; # everything up to the first '_';
	
	
	
	my $index_p;
	my $resource_p;
	
	# if place already defined, hook the element to its place
	if (defined $element_registry{$flatres_name}) {
	  $index_p=$element_registry{$flatres_name};
	  $resource_p=$self->_add_element($index_p,$resource_name);
	} else {
	  $element_registry{$flatres_name}=$comp_p;
	  $index_p=$comp_p;	# by default, add elements at 'component' level
	  $resource_p=$self->_add_element($index_p,$flatres_name);
	  # note that here, the element is hooked with the 'flat' resource
	  # name. This is to avoid clashes between elements with the same
	  # name (eg. 'partitions_hdb' and 'partitions_hda' both are transformed
	  # into 'partitions'). Once the correct hook for the element
	  # has been found, the element is renamed.
	}
	
	
	# now we have in $index_p the pointer for adding at the
	# right (temporary) place this resource.
	# temporary==there might be another struct which lies between the resource
	# and the root.
	
	
	$resource_p->{'TYPE'}='resource'; # type: resource (interior element)
	
	# its values are the 'children',
	# and the type contains the 'grandchildren'
	
	# create children
	my $child_name;
	foreach $child_name (split (/\s+/,$flatres->{'VALUE'})) {
	  my $child_p=$self->_add_element($resource_p,$child_name);
	  $child_p->{'TYPE'}='resource'; # type: resource (interior element)
	  
	  # create grandchildren
	  my $grandchildren=$flatres->{'TYPE'};;
	  $grandchildren =~ s%list\:\s+(.+)%$1%; # remove leading 'list:'
	  
	  my $gc_name;
	  foreach $gc_name (split (/\s+/,$grandchildren)) {
	    
	    # substitute all the _$_ and register the child
	    # for get_matching_element
	    # code has to be changed if '_' is *not* a reserved char!
	    #
	    
	    my $upper_subst=$flatres_name;
	    if ($flatres_name =~ m%_%) {
	      $upper_subst =~ s%^([^_]+)(\_.+)$%$2%; # eg. partitions_hda_hda1 -> _hda_hda1
	    } else {
	      $upper_subst='';	# no '_' -> no substring..
	    }
	    
	    # remove _$_$.._$ from grandchild
	    $gc_name =~ s%^([^_]+)(\_.+)$%$1%; #again '_': reserved char
	    
	    my $rg_idx=$gc_name . $upper_subst .'_'. $child_name;
	    my $rg_ptr=$element_registry{$rg_idx};
	    
	    if (defined $rg_ptr) {
	      # the grandchild has already been registered because
	      # it was found on the resource list before
	      # Now move it to the right place
	      
	      my $gc_ptr=$self->_get_element_p($rg_ptr,$rg_idx);
	      $self->_rm_element($rg_ptr,$rg_idx);
	      # rename element (see above!)
	      $gc_ptr->{'NAME'}=$gc_name;
	      $self->_add_element_p($child_p,$gc_ptr);
	    }
	    
	    $element_registry{$rg_idx}=$child_p;
	    
	    
	  }
	}
	
	
	# drop the element from the processed resource list
	# and indicate that there has been a change to the resource list
	# (one complete iteration without changes: bug somewhere!)
	# (avoid to loop for ever)
	
	
	$resources_to_process{$flatres_name}=0;
	$changes=1;
	$nr_resources_process--;
	
      }
    }
    
  } until ($changes==0);
  
  
  foreach (keys(%{$comp})) {
    if ($resources_to_process{$_}==1) {
      warn "not processed: $_\n"; undef $ok;
    }
  }
  
  #  now process the non-structure resources...
  
  
  my $flatres_name;
  foreach $flatres_name (keys (%{$comp})) {

    # ignore LCFG internal ng_ and om_ resources    
    next if ($flatres_name =~ m%^(ng|om)_%);

    my $el_rg=$element_registry{$flatres_name};
    
    my $type=$comp->{$flatres_name}->{'TYPE'};
    
    next if (defined $type && $type =~ m%^list\:%);
    
    #
    # the non-structure remaining elements are either
    # 1. registered in %element_registry
    # 2. are root elements
    
    my $flatres = $comp->{$flatres_name};
    
    # if element is not registered, add it at the component level
    my $father_p = (defined $el_rg) ? $el_rg : $comp_p ;
    
    my $resource_name=$flatres_name;
    $resource_name =~ s%([^_]+)(\_.+)%$1%; # everything up to the first '_';
    
    my $resource_p=$self->_add_element($father_p,$resource_name);
    # add values
    $resource_p->{'VALUE'}=$flatres->{'VALUE'};
    $resource_p->{'TYPE'}=
      (defined $flatres->{'TYPE'}) ? $flatres->{'TYPE'} : "string"; 
    # default type: string
    $resource_p->{'DERIVE'}=$flatres->{'DERIVE'};
    $resource_p->{'CONTEXT'}=$flatres->{'CONTEXT'};
    
  }
  
  
  # assign to object variables
  $self->{'ELEMENT_REGISTRY'}=\%element_registry;
  
  
  return $ok;
}





=pod

=item _add_element($p,$name): $element

Adds the child element called $name and type $type to $p, returns reference to it.

=cut


sub _add_element {
  my ($self,$p,$name)=@_;

  my $e;

  my %element;
  $element{'NAME'}=$name;

  unless (defined $self->_add_element_p($p,\%element)) {
    return undef;
  }

  return \%element;
}

=pod

=item _rm_element($p,$name): $element

Removes the child element called $name from $p. Returns undef if not found

=cut


sub _rm_element {
  my ($self,$p,$name)=@_;

  if (exists $p->{'ELEMENTS'}->{$name}) { 
    delete $p->{'ELEMENTS'}->{$name};
    my @e_list;
    # redo the ordered list
    foreach (@{$p->{'E_LIST'}}) {
      push (@e_list,$_) unless ($_ eq $name);
    }
    $p->{'E_LIST'} = \@e_list;

    return 1;
  } else {
    warn "element $name does not exist";
    return undef; # not found
  }
}



=pod

=item _get_element_p($p,$name)

returns a reference to the child element named $name of $p.

=cut



sub _get_element_p {
  my ($self,$p,$name)=@_;

  if (exists $p->{'ELEMENTS'}->{$name}) { 
    return $p->{'ELEMENTS'}->{$name};
  } else {
    return undef;
  }
}


=pod

=item _add_element_p($p,$child_p)

adds a child element referenced by $child_p to $p.

=cut


sub _add_element_p {
  my ($self,$p, $child_p)=@_;
  
  my $name=$child_p->{'NAME'};
  if (exists $p->{'ELEMENTS'}->{$name}) { 
    warn "element $name already found";
    return undef;
  } else {
    $p->{'ELEMENTS'}->{$name}=$child_p;
    push (@{$p->{'E_LIST'}}, $name);
    return 1;
  }
}




=pod

=back

=head2 Public methods

=over 4

=item new($res): create new StructResources instance.

=cut

sub new {
  my $proto = shift;
  my $class = ref($proto) || $proto;
  my $self = {};
  bless($self,$class);

  if ($self->_initialize(@_)) {
    return $self;
  } else {
    undef $self;
    return undef;
  }
}



=pod

=item getpbyname($name):$p

returns a reference to the element $p located in the 
absolute path described in $name. (eg. '/a/b/c').

=cut


sub getpbyname {
  my ($self,$name)=@_;

  $name =~ s%^/(.*)$%$1%; # remove leading '/';
  my @names=split('/',$name);

  my $p=$self->{'STRUCTRES'};
  
  my $n;
  foreach $n (@names) {
    #search element called $n: has to be found in the child elements of $p
    if (exists $p->{'ELEMENTS'}->{$n}) { 
      # advance
      $p=$p->{'ELEMENTS'}->{$n};
      next;
    } else {
      # not found!
      warn "element $name not found";
      return undef;
    }
  }
  return $p;    # success
}





=pod

=head2 Internal data structure

=over 4

    
 Data structure: $p->{'NAME'}       string with name for this element
                     {'ELEMENTS'}   pointer to childs (hash keyed by element names)
                                    where each hash value is a pointer to the
                                    next $p pointer
                     {'E_LIST'} ordered name of elements
                     {'TYPE'}       can be 'resource', 'string', ... 
 
                     only for properties:
                     {'CONTEXT'}
                     {'DERIVE'}
                     {'VALUE'}
=cut



#------------------------------------------------------------
#                      Other doc
#------------------------------------------------------------

=pod

=head1 SEE ALSO

  LCFG::Resources

=head1 AUTHORS

  German Cancio <German.Cancio@cern.ch>

  Version: 1.0.10 : 18/03/03 14:38

=cut

1; ## END ##

